/* This test driver program reads an encoded record from a file         */
/* and decodes it and displays the results..                            */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxStreamFile.h"
#include "rtxmlsrc/rtSaxCParser.h"
#include "employee.h"

int main (int argc, char** argv)
{
   PersonnelRecord employee;
   OSXMLQName   qname;
   OSCTXT	ctxt;
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: reader [ -v ] [ -i <filename>] [ -notrace ]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context structure */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose) rtxSetDiag (&ctxt, TRUE);

   /* Init PDU data variable */

   Init_PersonnelRecord (&ctxt, &employee);

   /* Create input source object */

   stat = rtxStreamFileOpen (&ctxt, filename, OSRTSTRMF_INPUT);
   if (0 != stat) {
      printf ("Unable to open file input stream.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   rtSaxSetMaxErrors (&ctxt, 0);

   /* Check initial element tag on message */

   stat = rtXmlParseElemQName (&ctxt, &qname);
   if (0 == stat) {
      if (trace) {
         printf ("Initial element tag is %s\n", qname.ncName);
      }
   }
   else {
      printf ("parse initial tag failed\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   /* Validate */

   if (0 == stat)
      stat = XmlV_personnelRecord (&ctxt);

   if (0 == stat) {
      if (trace) {
         printf ("Message is valid\n");
      }
   }
   else {
      printf ("validation failed\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   /* Call compiler generated decode function */

   stat = rtxStreamReset (&ctxt);

   if (stat == 0)
      stat = XmlD_personnelRecord (&ctxt, &employee);

   if (stat == 0) {
      if (trace) {
         printf ("Decode was successful\n");
         printf ("Decoded record:\n");
         Print_PersonnelRecord ("employee", &employee);
      }
   }
   else {
      printf ("decode failed\n");
      rtxErrPrint (&ctxt);
      return -1;
   }

   rtxStreamClose (&ctxt);
   rtxFreeContext (&ctxt);

   return 0;
}

