/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "po.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   PurchaseOrderType po;
   Items_item*  pItem;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN];
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;
   

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [ -v ] [ -o <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
   if (verbose)
      rtxSetDiag (&ctxt, 1);

   /* Populate structure of generated type */

   Init_PurchaseOrderType (&ctxt, &po);

   po.orderDate.value = (const OSUTF8CHAR*) "2003-04-01";
   po.m.orderDatePresent = TRUE;

   po.shipTo.name.value = (const OSUTF8CHAR*) "Joe Smith";
   po.shipTo.street.value = (const OSUTF8CHAR*) "1 Morning Light Rd";
   po.shipTo.city.value = (const OSUTF8CHAR*) "Glenmoore";
   po.shipTo.state.value = (const OSUTF8CHAR*) "PA";
   po.shipTo.zip = 19343;

   po.billTo.name.value = (const OSUTF8CHAR*) "Bill Jones";
   po.billTo.street.value = (const OSUTF8CHAR*) "102 Pickering Way, Suite 506";
   po.billTo.city.value = (const OSUTF8CHAR*) "Exton";
   po.billTo.state.value = (const OSUTF8CHAR*) "PA";
   po.billTo.zip = 19341;

   po.m.commentPresent = TRUE;
   po.comment.value = (const OSUTF8CHAR*) "Need this ASAP!";

   pItem = OSRTALLOCTYPE (&ctxt, Items_item);
   Init_Items_item (&ctxt, pItem);
   pItem->productName.value = (const OSUTF8CHAR*) "Acme Widget";
   pItem->quantity = 50;
   pItem->USPrice = 200;
   pItem->partNum.value = (const OSUTF8CHAR*) "111-AA";
   rtxDListAppend (&ctxt, &po.items._elemList, pItem);

   pItem = OSRTALLOCTYPE (&ctxt, Items_item);
   Init_Items_item (&ctxt, pItem);
   pItem->productName.value = (const OSUTF8CHAR*) "Wizbang Gadget";
   pItem->quantity = 99;
   pItem->USPrice = 777;
   pItem->m.commentPresent = TRUE;
   pItem->comment.value = 
      (const OSUTF8CHAR*) "I would like these in the color red";
   pItem->m.shipDatePresent = TRUE;
   pItem->shipDate.value = (const OSUTF8CHAR*) "2003-03-26";
   pItem->partNum.value = (const OSUTF8CHAR*) "222-BB";
   rtxDListAppend (&ctxt, &po.items._elemList, pItem);
   
   /* Encode */

   stat = rtXmlSetEncBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   if (0 == stat)
      stat = XmlE_purchaseOrder (&ctxt, &po);

   if (0 == stat) {
      if (trace) {
         printf ("encoded XML message:\n");
         puts ((char*)msgbuf);
         printf ("\n");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
  
   /* Write the encoded message out to the output file */

  stat = rtXmlWriteToFile (&ctxt, filename);
  if (0 != stat) {
     printf ("Write to file %s failed\n", filename);
     rtxErrPrint (&ctxt);
  } 

   rtxFreeContext (&ctxt);
   return 0;
}
