package com.objsys.xbinder.sample.rss;

import gen.rss.RssChannel;
import gen.rss.RssItem;
import gen.rss.RssItem_1;
import gen.rss.Rss_CC;
import gen.rss.Rss_ELEM;

import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.Duration;
import javax.xml.datatype.XMLGregorianCalendar;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.method.ScrollingMovementMethod;
import android.widget.TextView;

public class RSSReader extends Activity
{
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        	    
        final TextView tv = new TextView(this);
        tv.setMovementMethod(new ScrollingMovementMethod()); 
        tv.setBackgroundColor(0xFF008B45);
        tv.setHorizontallyScrolling(true);
        tv.setHorizontalScrollBarEnabled(true);
        tv.setVerticalScrollBarEnabled(true);

        setContentView(tv);
                
        tv.setText("Obtaining feed...");
        
        /* This handler receives a message with the decoded RSS data and
         * updates the UI.
         */
        final Handler feedHandler = new Handler() {
        	
			@Override
			//We expect msg.obj to hold the decoded XML message (ie Rss_CC).
			public void handleMessage(Message msg) {
				tv.setText("CBS News feed:\n");
				
				Rss_CC rss = (Rss_CC) msg.obj;
                Rss_ELEM rssElem = rss.getRss();
                RssChannel channel = rssElem.getChannel();
                List<RssItem> items = channel.getItem();
                for (RssItem rssItem : items) {
                	//Look for title or description among list of optional elements
                    String description = null;
                    String title = null;
        			List<RssItem_1> itemChildren = rssItem.getRssItem_1();
        			for (RssItem_1 item : itemChildren) {
        				if ( item.isSetDescription() ) {
        					description = item.getDescription();
        				}
        				else if ( item.isSetTitle()) {
        					title = item.getTitle();
        				}
        			}
        	        if ( title != null ) {
        	        	tv.append(title);
        	        	tv.append("\n");
        	        }
        	        if ( description != null ) {
        	        	tv.append( description );
        	        	tv.append("\n\n\n");
        	        }
        	        else tv.append("\n");
        		}  
                
                //Just some code to demonstrate XMLGregorianCalendar and Duration
                //(which are not a part of Android) are working
                try {
                	DatatypeFactory dtFactory = DatatypeFactory.newInstance();
                	XMLGregorianCalendar now = dtFactory.newXMLGregorianCalendar(
                			new GregorianCalendar() );
                	Duration d = dtFactory.newDuration("P10D");
                	tv.append("\nUpdated: " + now.toString());
                }
                catch (RuntimeException e) { throw e; }
                catch (Exception e) {
                	throw new RuntimeException(e);
				    }
			}        	
        };
        
        /* Runnable to actually get and decode the RSS feed.  We do this work
         * outside the UI thread.
         */
        Runnable feedRunnable = new Runnable() {
        	public void run() {
        		//get the feed and then send a message to the handler
        		//so that the UI thread can use the decoded data to update
        		//the UI
                Rss_CC rss = RSSClient.getFeed("http://feeds.cbsnews.com/CBSNewsMain?format=xml");
                
                Message msg = feedHandler.obtainMessage();
                msg.obj = rss;
                feedHandler.sendMessage(msg);                
        	}
        };
        
        Thread feedThread = new Thread(feedRunnable);
        feedThread.start();
    }
}
