import com.test.*;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.util.List;
import com.objsys.xbinder.runtime.*;

/**
 * This test program encodes a data record and writes the encoded result to an output file.
 */
public class Writer {

   private static DatatypeFactory dtFactory;

   static {
      try {
         dtFactory = DatatypeFactory.newInstance();
      }
      catch ( javax.xml.datatype.DatatypeConfigurationException dce ) 
      { throw new RuntimeException(dce); }
   }

   private static String outputFile = "message.xml";
   private static String charset = "UTF-8";     //default is UTF-8
   private static boolean trace = true;

   public static void main(String args[]) throws Exception 
   {
      //process command line to get the name of the output file
      //process command line to get the name of the input file
      for(int arg = 0; arg < args.length; arg++) {
         if ( args[arg].equals("-o")) {
           outputFile = args[++arg];
         }
         else if ( args[arg].equals("-notrace" ) ) trace = false;
         else {
            System.out.println("Usage: java Writer [-o <filename>] [-notrace]");
            System.out.println("  -o <filename> specifies the output file");
            System.out.println("  -notrace turns off echoing encoded data to stdout");
            System.exit(1);
         }
      }

      //create _CC object
      MakeCall_CC root = new MakeCall_CC();
      java.util.List<String> schemaLocation = root._xbContext.getSchemaLocation();
      schemaLocation.add("http://www.ecma.ch/standards/ecma-323/csta/ed2");
      schemaLocation.add("make-call.xsd");

      MakeCall_ELEM makeCall = new MakeCall_ELEM();
      root.setMakeCall(makeCall);

      DeviceID callingDevice = new DeviceID();
      makeCall.setCallingDevice(callingDevice);
   
      callingDevice.setValue("22343");

      DeviceID calledDirectoryNumber = new DeviceID();
      makeCall.setCalledDirectoryNumber(calledDirectoryNumber);

      calledDirectoryNumber.setValue("18005551212");
      
      makeCall.setAccountCode(new XBByteArray( new byte[] {0,1} ) );
      makeCall.setAuthCode(new XBByteArray( new byte[] {0,1} ) );
      makeCall.setAutoOriginate(AutoOriginate.prompt);
      
      CorrelatorData correlatorData = new CorrelatorData();
      makeCall.setCorrelatorData(correlatorData);

      correlatorData.setString(new XBByteArray( new byte[] {0,1} ) );

      UserData userData = new UserData();
      makeCall.setUserData(userData);

      userData.setString(new XBByteArray( new byte[] {0,1} ) );

      CallCharacteristics callCharacteristics = new CallCharacteristics();
      makeCall.setCallCharacteristics(callCharacteristics);

      callCharacteristics.setPriorityCall(true);

      MediaCallCharacteristics mediaCallCharacteristics = new MediaCallCharacteristics();
      makeCall.setMediaCallCharacteristics(mediaCallCharacteristics);

      MediaClass mediaClass = new MediaClass();
      mediaCallCharacteristics.setMediaClass(mediaClass);

      mediaClass.setVoice(true);

      ConnectionInformation callingConnectionInfo = new ConnectionInformation();
      makeCall.setCallingConnectionInfo(callingConnectionInfo);

      callingConnectionInfo.setFlowDirection(ConnectionInformation_flowDirection.transmitAndReceive);
      callingConnectionInfo.setNumberOfChannels(2);

      CSTACommonArguments extensions = new CSTACommonArguments();
      makeCall.setExtensions(extensions);

      CSTASecurityData security = new CSTASecurityData();
      extensions.setSecurity(security);

      security.setMessageSequenceNumber(123);

      SecurityInfo securityInfo = new SecurityInfo();
      security.setSecurityInfo(securityInfo);
      securityInfo.setString(new XBByteArray( new byte[] {0,1} ) );

      CSTAPrivateData privateData = new CSTAPrivateData();
      extensions.setPrivateData(privateData);

      privateData.setString(new XBByteArray( new byte[] {0,1} ) );


      //Encode data
      ByteArrayOutputStream bos = new ByteArrayOutputStream(32768);

      XBXmlEncoder encoder;
      encoder = new XBXmlEncoder(bos, charset);
      encoder.setNamespaces(_make_call.namespaceContext);
      root.encodeDocument(encoder);
      encoder.close();

      if ( trace ) {
         //write encoded data to System.out
         System.out.println("Encoded data:\n");
         bos.writeTo(System.out);
         System.out.println();
      }

      //write encoded data to output file
      FileOutputStream fout = new FileOutputStream(outputFile);
      bos.writeTo(fout);
      fout.close();
   }
}
